@@ -61,7 +61,9 @@ obs_set = [
      transform([[0.2, 0.8], [0.8, 0.8], HORIZONTAL]),
      transform([[0.2, 0.2], [0.2, 0.8], VERTICAL]),
      transform([[0.8, 0.2], [0.8, 0.8], VERTICAL]),
-     transform([[0.2, 0.2], [0.5, 0.5], CIRCULAR])]
+     transform([[0.2, 0.2], [0.5, 0.5], CIRCULAR]),
+     transform([[0.3, 0.3], [0.5, 0.5], CIRCULAR]),
+     transform([[0.4, 0.4], [0.5, 0.5], CIRCULAR])]
 ]
 
 goal_set = [denormalize_position([0.2, 0.2], W, H),
@@ -69,7 +71,7 @@ goal_set = [denormalize_position([0.2, 0.2], W, H),
             denormalize_position([0.8, 0.8], W, H),
             denormalize_position([0.8, 0.2], W, H),
             denormalize_position([0.0756, 0.5], W, H),
-            denormalize_position([0.5, 0.5], W, H),
+            denormalize_position([0.5, 0.0756], W, H),
             denormalize_position([0.924, 0.5], W, H),
             denormalize_position([0.5, 0.924], W, H)]
 
@@ -81,3 +83,13 @@ for obs in obs_set:
         task_dict['OBSTACLE_POSITIONS'] = obs
         task_dict['Goal'] = goal
         config_set.append(task_dict)
+
+non_sta_config_set = []
+for i in range(5):
+    for j in range(5):
+        task_dict = {}
+        task_dict['OBSTACLE_POSITIONS'] = obs_set[0]
+        task_dict['Goal'] = goal_set[0]
+        task_dict['uncertainty'] = j
+        task_dict['dynamics'] = 0.01 * i
+        non_sta_config_set.append(task_dict)
\ No newline at end of file
