@@ -67,7 +67,7 @@ class NavigationEnvDefault(gym.Env, EzPickle):
         self.observation_meta_data = {
             'position': gym.spaces.Box(np.array([0, 0]), np.array([W, H]), dtype=np.float32),
             'distance': gym.spaces.Box(np.array([0]), np.array([np.sqrt(W ** 2 + H ** 2)]), dtype=np.float32),
-            'lidar': gym.spaces.Box(low=0, high=1, shape=(16,)),
+            'lidar': gym.spaces.Box(low=0, high=1, shape=(8,)),
             'energy': gym.spaces.Box(low=0, high=1, shape=(1,)),
             'obstacle_speed': gym.spaces.Box(low=-1, high=1, shape=(len(task_args['OBSTACLE_POSITIONS']),)),
             'obstacle_position': gym.spaces.Box(low=0, high=1, shape=(2 * len(task_args['OBSTACLE_POSITIONS']),))
@@ -94,13 +94,14 @@ class NavigationEnvDefault(gym.Env, EzPickle):
         self.goal = None
         self.obs_range_plt = None
         self.max_obs_range = max_obs_range
-        self.num_beams = 16
+        self.num_beams = 8
         self.lidar = None
         self.drawlist = None
         self.achieve_goal = False
         self.strike_by_obstacle = False
         self.dynamics = initial_speed
-        self.energy = 1
+        self.energy = 0.2
+
         p1 = (0.75, 0.5)
         p2 = (W - 0.5, 0.5)
         self.sky_polys = [[p1, p2, (p2[0], H-0.5), (p1[0], H-0.5)]]
@@ -116,7 +117,7 @@ class NavigationEnvDefault(gym.Env, EzPickle):
 
     @property
     def drone_start_pos(self):
-        return denormalize_position(np.array([0.5, 0.5]), W ,H)
+        return denormalize_position(np.random.random(2) * 0.8 + 0.1, W ,H)
 
     @property
     def observation_space(self):
@@ -221,11 +222,11 @@ class NavigationEnvDefault(gym.Env, EzPickle):
                 self.game_over = False
             else:
                 break
-        return self.drone_start_pos
+        return self.drone.position
 
     def _build_goal(self):
         goal_pos = self.task_args['Goal']
-        self.goal = self.world.CreateStaticBody(position=goal_pos, angle=0.0,
+        self.goal = self.world.CreateDynamicBody(position=goal_pos, angle=0.0,
                                                 fixtures=fixtureDef(shape=polygonShape(vertices=[(x / SCALE, y / SCALE) for x, y in DRONE_POLY]),
                                                                     density=5.0, friction=0.1, categoryBits=0x002,
                                                                     maskBits=0x0010, restitution=0.0))
@@ -264,12 +265,13 @@ class NavigationEnvDefault(gym.Env, EzPickle):
             else:
                 self.scores.append(0)
 
+
         self.game_over = False
         self.prev_shaping = None
         self.achieve_goal = False
         self.strike_by_obstacle = False
         # timer
-        self.energy = 1
+        self.energy = 0.2
         # clean up objects in the Box 2D world
         self._destroy()
         # create lidar objects
@@ -320,7 +322,7 @@ class NavigationEnvDefault(gym.Env, EzPickle):
                 reward = 1
             info['is_success'] = self.achieve_goal
             info['energy'] = self.energy
-            info['episode'] = {'r': reward, 'l': (1 - self.energy) * 1000}
+            info['episode'] = {'r': reward, 'l': (0.15 - self.energy) * 1000}
         obs = np.copy(self.array_observation())
         self.obs_queue.append(obs)
         return obs, reward, done, info
@@ -362,10 +364,13 @@ class NavigationEnvDefault(gym.Env, EzPickle):
 
 
 class NavigationEnvAcc(NavigationEnvDefault):
-    def __init__(self, obstacles_args, max_obs_range=3, max_speed=5, initial_speed=2, **kwargs):
+    def __init__(self, obstacles_args, max_obs_range=3, max_speed=2, initial_speed=1, **kwargs):
         super().__init__(obstacles_args, max_obs_range, max_speed, initial_speed, )
-        self.observation_meta_data['velocity'] = gym.spaces.Box(np.array([-3, -3]), np.array([3, 3]), dtype=np.float32)
+        self.observation_meta_data['velocity'] = gym.spaces.Box(np.array([-1, -1]), np.array([1, 1]), dtype=np.float32)
         self.observation_meta_data_keys.append('velocity')
+        self.timesteps = 0
+        self.prev_distance = 0
+        self.randomize_goal = False
 
     def dict_observation(self):
         dict_obs = super().dict_observation()
@@ -373,24 +378,27 @@ class NavigationEnvAcc(NavigationEnvDefault):
         dict_obs['velocity'] = velocity
         return dict_obs
 
+    @property
+    def distance(self):
+        return np.linalg.norm(self.goal.position - self.drone.position)
+
     def step(self, action: np.iterable):
         action = np.asarray(action, dtype=np.float64)
         before_pos = np.asarray(self.drone.position).copy()
-
+        self.timesteps += 1
         self.action = action
         # mass == 1 impulse == action
-        self.drone.ApplyForce((action[0] * 5, action[1] * 5), self.drone.position, wake=True)
+        self.drone.ApplyForce((action[0], action[1]), self.drone.position, wake=True)
         self.world.Step(1.0 / FPS, 6 * 30, 2 * 30)
         v = self.drone.linearVelocity
-        v = np.clip(v, a_min=[-3, -3], a_max=[3, 3])
+        v = np.clip(v, a_min=-1, a_max=1)
         self.drone.linearVelocity.Set(v[0], v[1]) # clip velocity
-
         self.obstacles.step()
         self.energy -= 1e-3
         pos = np.array(self.drone.position)
         self._observe_lidar(pos)
-        reward = np.linalg.norm(self.goal.position - before_pos) - np.linalg.norm(self.goal.position - self.drone.position)
-
+        reward = self.prev_distance - self.distance
+        self.prev_distance = self.distance
         done = self.game_over
 
         info = {}
@@ -401,22 +409,35 @@ class NavigationEnvAcc(NavigationEnvDefault):
                 reward = 1
             info['is_success'] = self.achieve_goal
             info['energy'] = self.energy
-            info['episode'] = {'r': reward, 'l': (1 - self.energy) * 1000}
+            info['episode'] = {'r': reward, 'l': (0.2 - self.energy) * 1000}
 
         obs = np.copy(self.array_observation())
         self.obs_queue.append(obs)
         return obs, reward, done, info
 
+    def reset(self):
+        state = super().reset()
+        self.prev_distance = self.distance
+        self.timesteps = 0
+        return state
+
+    def randomize_goal(self):
+        theta = np.pi * 2 * np.random.random()
+        goal = denormalize_position(np.array([np.cos(theta), np.sin(theta)]) * 0.424 + np.array([0.5, 0.5]),
+                                    W, H)
+        self.task_args['Goal'] = goal
+        self.goal.position.Set(goal[0], goal[1])
+
 
 class NavigationEnvAccLidarObs(NavigationEnvAcc):
-    def __init__(self, obstacles_args, max_obs_range=3,  max_speed=5, initial_speed=2, **kwargs):
-        super().__init__(obstacles_args, max_obs_range, max_speed, initial_speed, )
+    def __init__(self, task_args, max_obs_range=3, max_speed=2, initial_speed=1, **kwargs):
+        super().__init__(task_args, max_obs_range, max_speed, initial_speed, **kwargs)
         self.observation_meta_data = {
             'position': gym.spaces.Box(np.array([0, 0]), np.array([W, H]), dtype=np.float32),
             'distance': gym.spaces.Box(np.array([0]), np.array([np.sqrt(W ** 2 + H ** 2)]), dtype=np.float32),
-            'lidar': gym.spaces.Box(low=0, high=1, shape=(16,)),
+            'lidar': gym.spaces.Box(low=0, high=1, shape=(8,)),
             'energy': gym.spaces.Box(low=0, high=1, shape=(1,)),
-            'velocity': gym.spaces.Box(np.array([-3, -3]), np.array([3, 3]), dtype=np.float32)
+            'velocity': gym.spaces.Box(np.array([-1, -1]), np.array([1, 1]), dtype=np.float32)
         }
         self.observation_meta_data_keys = ['position', 'lidar', 'energy', 'distance', 'velocity']
 
@@ -424,10 +445,51 @@ class NavigationEnvAccLidarObs(NavigationEnvAcc):
         position = normalize_position(self.drone.position, W, H)
         distance = np.linalg.norm(normalize_position(self.drone.position, W, H) - (normalize_position(self.goal.position, W, H)))
         lidar = [l.fraction for l in self.lidar]
+        velocity = self.drone.linearVelocity
         dict_obs = {
             'position': position,
             'distance': distance,
             'lidar': lidar,
-            'energy': self.energy
+            'energy': self.energy,
+            'velocity': velocity
         }
-        return dict_obs
\ No newline at end of file
+        return dict_obs
+
+class NonStationaryNavigation(NavigationEnvAccLidarObs):
+    def __init__(self, task_args, max_obs_range=3, max_speed=2, initial_speed=1, **kwargs):
+        super().__init__(task_args, max_obs_range, max_speed, initial_speed, **kwargs)
+        self.theta = 0
+        self.dynamics = task_args['dynamics']
+        self.uncertainty = task_args['uncertainty']
+
+    def _build_goal(self):
+        goal_pos = self.task_args['Goal']
+        self.goal = self.world.CreateDynamicBody(position=goal_pos, angle=0.0,
+                                                 fixtures=fixtureDef(shape=polygonShape(
+                                                     vertices=[(x / SCALE, y / SCALE) for x, y in DRONE_POLY]),
+                                                                     density=5.0, friction=0.1, categoryBits=0x002,
+                                                                     maskBits=0x0010, restitution=0.0))
+        self.goal.color1 = (0., 0.5, 0)
+        self.goal.color2 = (0., 0.5, 0)
+        self.theta += np.pi * 2 * np.random.random()
+        goal = denormalize_position(np.array([np.cos(self.theta), np.sin(self.theta)]) * 0.424 + np.array([0.5, 0.5]), W, H)
+        self.task_args['Goal'] = goal
+        self.goal.position.Set(goal[0], goal[1])
+
+    def step(self, action):
+        state, reward, done, info = super().step(action)
+        if np.random.random() < self.dynamics:
+            self.moving_goal()
+        return state, reward, done, info
+
+    def moving_goal(self):
+        self.theta += np.pi / 180 * (np.random.normal(0, self.uncertainty) * 5)
+        goal = denormalize_position((np.array([np.cos(self.theta), np.sin(self.theta)]) * (0.424)) + np.array([0.5, 0.5]), W, H)
+        self.task_args['Goal'] = goal
+        self.goal.position.Set(goal[0], goal[1])
+        self.prev_distance = self.distance
+        return
+
+    @property
+    def drone_start_pos(self):
+        return denormalize_position([0.5, 0.5], W ,H)
\ No newline at end of file
